/* test.c */
#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <unistd.h>
#include <sys/mman.h>
#include <sys/ioctl.h>
#include <sys/fcntl.h>
#include <signal.h>
#include <asm/sigcontext.h>
#include <asm/ucontext.h>
#include "linux/perfctr.h"
#define PAGE_SIZE	4096

int fd;
volatile const struct vperfctr_state *kstate;
struct vperfctr_icontrol icontrol;

void do_open(void)
{
    fd = open("/proc/self/perfctr", O_RDONLY);
    if( fd < 0 ) {
	perror("open");
	exit(1);
    }
    kstate = mmap(NULL, PAGE_SIZE, PROT_READ, MAP_SHARED, fd, 0);
    if( kstate == MAP_FAILED ) {
	perror("mmap");
	exit(1);
    }
}

void on_sigio(int sig, siginfo_t *si, void *puc)
{
    printf(__FUNCTION__ ": sig %d, code %#x\n", sig, si->si_code);
    if( sig != SIGIO || si->si_code != 0x4711 )
	return;
    printf(__FUNCTION__ ": pc %#lx\n", kstate->pc);
    if( ioctl(fd, VPERFCTR_IRESUME, &icontrol.ctrs) < 0 ) {
	perror("iresume");
	exit(1);
    }
}

void do_sigaction(void)
{
    struct sigaction sa;
    memset(&sa, 0, sizeof sa);
    sa.sa_sigaction = on_sigio;
    sa.sa_flags = SA_SIGINFO;
    if( sigaction(SIGIO, &sa, NULL) < 0 ) {
	perror("sigaction");
	exit(1);
    }
}

void do_icontrol(void)
{
    memset(&icontrol, 0, sizeof icontrol);
    icontrol.control.evntsel[0] = 0xC1 | (1 << 16) | (1 << 22) | (1<<20);
    icontrol.ctrs.ctr[1] = -25;
    icontrol.si_signo = SIGIO;
    icontrol.si_code = 0x4711;
    if( ioctl(fd, VPERFCTR_ICONTROL, &icontrol) < 0 ) {
	perror("icontrol");
	exit(1);
    }
}

#define N 150
double v[N], w[N];
double it;

void do_dotprod(void)
{
    int i;
    double sum;

    sum = 0.0;
    for(i = 0; i < N; ++i)
	sum += v[i] * w[i];
    it = sum;
}

int main(void)
{
    printf("A\n");
    do_sigaction();
    printf("B\n");
    do_open();
    printf("C\n");
    do_icontrol();
    printf("D\n");
    do_dotprod();
    printf("%#016llx\n", kstate->sum.ctr[0]);
    printf("%#016llx\n", kstate->sum.ctr[1]);
    printf("%d\n", kstate->si_signo);
    printf("%d\n", kstate->iresume.ctrs.ctr[0]);
    return 0;
}
